/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var Plugins = function()
{
	var plugins = {};
	
	var Permission = {};
	
	/**
		Initialize the page.
	*/
	plugins.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', plugins.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', plugins.validatePlugin);
			if (Doc.element('settings.annotations'))
			{
				TabControl.addTabActivateListener('settings.annotations', Annotations.autoLoadEditFrame);
			}
			
			// Job agents
			plugins.agentSettings = Data.json('page-data', 'agent-settings');

			Events.addEventHandler('agents', 'click', plugins.agentsOnClick);
			Events.addEventHandler('agents', 'base-selected', plugins.agentsOnClick);
			Events.addEventHandler('btnRemoveAgents', 'click', plugins.hideAgentSettings);
			
			Events.addEventHandler('agentMaxMemory', 'blur', plugins.maxMemoryOnBlur);
			Events.addEventHandler('useDefaultMaxMemory', 'click', plugins.useDefaultMaxMemoryOnClick)
			Events.addEventHandler('agentTrustedDefault', 'click', plugins.trustedOnClick);
			Events.addEventHandler('agentTrustedYes', 'click', plugins.trustedOnClick);
			Events.addEventHandler('agentTrustedNo', 'click', plugins.trustedOnClick);
			Events.addEventHandler('agentPriorityBoost', 'keypress', Events.integerOnly);
			Events.addEventHandler('agentPriorityBoost', 'blur', plugins.priorityBoostOnBlur);

			var selectedAgent = Data.get('page-data', 'select-agent');
			if (selectedAgent) plugins.selectAgent(selectedAgent);
			
			// Permissions
			Events.addEventHandler('permissionsYes', 'click', plugins.usePermissionsOnClick);
			Events.addEventHandler('permissionsNo', 'click', plugins.usePermissionsOnClick);
			Events.addEventHandler('permissionsAuto', 'click', plugins.usePermissionsOnClick);
			
			Events.addEventHandler('itemTypes', 'click', plugins.itemTypesOnClick);
			
			Events.addEventHandler('grant_create', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_read', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_use', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_write', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_delete', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_set_owner', 'click', plugins.permissionOnClick);
			Events.addEventHandler('grant_set_permission', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_create', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_read', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_use', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_write', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_delete', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_set_owner', 'click', plugins.permissionOnClick);
			Events.addEventHandler('deny_set_permission', 'click', plugins.permissionOnClick);
			
			Permission.CREATE = Data.int('page-data', 'create');
			Permission.READ = Data.int('page-data', 'read');
			Permission.USE = Data.int('page-data', 'use');
			Permission.WRITE = Data.int('page-data', 'write');
			Permission.DELETE = Data.int('page-data', 'delete');
			Permission.SET_OWNER = Data.int('page-data', 'set-owner');
			Permission.SET_PERMISSION = Data.int('page-data', 'set-permission');
			
			Buttons.addClickHandler('btnSetRequestedPermissions', plugins.setRequestedPermissions);
			
			plugins.initPermissionLetters();
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'PLUGINDEFINITION', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnNewConfiguration', plugins.newConfiguration, attributes);
			Buttons.addClickHandler('btnToggleDisabled', plugins.toggleDisabled);

			if (Doc.element('main.annotations'))
			{
				TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			}
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'PLUGINDEFINITION'};
			var tableAttributes = {'table-id': 'plugins'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}


	// Add event handlers to the 'configure' and copy icons
	plugins.initElements = function(element, autoInit)
	{
		if (autoInit == 'new-config')
		{
			Buttons.addClickHandler(element, plugins.newConfiguration);
		}
		else if (autoInit == 'configure-plugin')
		{
			Buttons.addClickHandler(element, plugins.configurePlugin);
		}
		else if (autoInit == 'configure-agent')
		{
			Buttons.addClickHandler(element, plugins.configureAgent);
		}
	}

	plugins.newConfiguration = function(event)
	{
		var pluginId = Data.get(event.currentTarget, 'item-id');
		Items.newItem('PLUGINCONFIGURATION', '&plugindefinition_id='+pluginId);
	}
	
	/*
		Run the plug-in configuration wizard
	*/
	plugins.configurePlugin = function(event)
	{
		var configId = Data.int(event.currentTarget, 'item-id');
		var url = App.getRoot() + 'common/plugin/index.jsp?ID='+App.getSessionId();
		url += '&cmd=ConfigurePlugin';
		url += '&pluginconfiguration_id='+configId;
		Dialogs.openPopup(url, 'ConfigurePlugin', 750, 500);
	}

	plugins.configureAgent = function(event)
	{
		var pluginId = Data.get('page-data', 'item-id');
		var agentId = Data.get(event.currentTarget, 'item-id');
		Items.editItem('PLUGINDEFINITION', pluginId, '&jobagent_id='+agentId);
	}
	
	plugins.toggleDisabled = function(event)
	{
		var enable = Data.int(event.currentTarget, 'enable');
		var tree = window.parent.frames['tree'];
		if (tree && tree.Tree)
		{
			var iconName = enable ? 'Plugin' : 'PluginDisabled';
			var className = Data.get(event.currentTarget, 'plugin-class');
			tree.Tree.setIcon(className, iconName);
		}
		var pluginId = Data.get('page-data', 'item-id');
		var iframe = Data.get('page-data', 'iframe');
		
		var cmd = enable ? 'EnableItem' : 'DisableItem';
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=' + (enable ? 'EnableItem' : 'DisableItem');
		url += '&item_id='+pluginId+'&iframe='+iframe;
		location.replace(url);
	}
	
	plugins.validatePlugin = function()
	{
		var frm = document.forms['plugin'];
		if (Strings.trim(frm.className.value) == '')
		{
			Forms.showNotification(frm.className, 'You must enter a class name');
			return false;
		}
		return true;
	}
	
	plugins.save = function()
	{
		var frm = document.forms['plugin'];
		if (TabControl.validateActiveTab('settings'))
		{
			Link.exportActions('agents');
			var agents = Link.getIdsInList(frm.agents, 'JOBAGENT');
			
			for (var i = 0; i < agents.length; i++)
			{
				var agentId = agents[i];
				var settings = plugins.getAgentSettings(agentId);
				if (settings)
				{
					if (settings.maxMemory)
					{
						Forms.addHidden(frm, 'maxMemory.'+agentId, settings.maxMemory);
					}
					Forms.addHidden(frm, 'trusted.'+agentId, settings.trusted);					
					Forms.addHidden(frm, 'priorityBoost.'+agentId, settings.priorityBoost);
				}
			}
			Forms.addHidden(frm, 'allAgents', agents.join(','));
			if (window.Annotations)
			{
				Annotations.saveModifiedAnnotationsToForm(frm);
			}
			frm.submit();
		}
	}


	plugins.getAgentSettings = function(agentId)
	{
		for (var i = 0; i < plugins.agentSettings.length; i++)
		{
			if (plugins.agentSettings[i].agentId == agentId)
			{
				return plugins.agentSettings[i];
			}
		}
		return null;
	}

	plugins.selectAgent = function(agentId)
	{
		var frm = document.forms['plugin'];
		for (var i = 0; i < frm.agents.length; i++)
		{
			if (frm.agents[i].item.id == agentId)
			{
				frm.agents.selectedIndex = i;
				plugins.agentsOnClick();
				return;
			}
		}
	}

	plugins.maxMemoryOnBlur = function()
	{
		var frm = document.forms['plugin'];
		var agentId = frm.agents[frm.agents.selectedIndex].item.id;
		var settings = plugins.getAgentSettings(agentId);
		if (settings)
		{
			settings.maxMemory = frm.agentMaxMemory.value;
		}
	}
	
	plugins.useDefaultMaxMemoryOnClick = function()
	{
		var frm = document.forms['plugin'];
		var agentId = frm.agents[frm.agents.selectedIndex].item.id;
		var settings = plugins.getAgentSettings(agentId);
		if (settings)
		{
			var useDefault = frm.useDefaultMaxMemory.checked;
			
			settings.maxMemory = useDefault ? null : frm.agentMaxMemory.value;
			frm.agentMaxMemory.disabled = useDefault;
			if (!frm.agentMaxMemory.disabled) frm.agentMaxMemory.focus();
		}
	}

	
	plugins.trustedOnClick = function()
	{
		var frm = document.forms['plugin'];
		var agentId = frm.agents[frm.agents.selectedIndex].item.id;
		var settings = plugins.getAgentSettings(agentId);
		
		if (settings)
		{
			var selected = Forms.getCheckedRadio(frm.agentTrusted);
			if (selected != null)
			{
				settings.trusted = selected.value;
			}
		}
	}

	
	plugins.priorityBoostOnBlur = function()
	{
		var frm = document.forms['plugin'];
		var agentId = frm.agents[frm.agents.selectedIndex].item.id;
		var settings = plugins.getAgentSettings(agentId);
		if (settings)
		{
			var priorityBoost = parseInt(frm.agentPriorityBoost.value);
			if (priorityBoost < 0 || priorityBoost > 10)
			{
				Forms.showNotification(frm.agentPriorityBoost, 'The priority boost must be between 0 and 10');
				return;
			}
			settings.priorityBoost = priorityBoost;
		}
	}
	
	
	plugins.agentsOnClick = function()
	{
		var frm = document.forms['plugin'];
		var settings;
		if (frm.agents.selectedIndex >= 0)
		{
			var item = frm.agents[frm.agents.selectedIndex].item;
			var agentId = item ? item.id : null;
			settings = plugins.getAgentSettings(agentId);
		}

		if (settings)
		{
			Doc.show('agentsettings');
			
			// Set max memory
			frm.useDefaultMaxMemory.checked = !settings.maxMemory;
			frm.agentMaxMemory.value = settings.maxMemory || '';
			frm.agentMaxMemory.disabled = frm.useDefaultMaxMemory.checked;

			var defaultMaxMemory = Doc.element('defaultMaxMemory');
			defaultMaxMemory.innerHTML = settings.defaultMaxMemory || '<i>not specified</i>';
			
			// Set trusted
			Forms.checkRadio(frm.agentTrusted, settings.trusted);
			var defaultTrusted = Doc.element('defaultTrusted');
			defaultTrusted.innerHTML = Forms.getCheckedRadio(frm.trusted).value == '0' ? 'no' : 'yes';

			// Set priority boost
			frm.agentPriorityBoost.value = settings.priorityBoost;
		}
		else
		{
			Doc.hide('agentsettings');
		}

	}

	
	plugins.hideAgentSettings = function()
	{
		Doc.hide('agentsettings');
	}

	plugins.usePermissionsOnClick = function()
	{
		var frm = document.forms['plugin'];
		var use = Forms.getCheckedRadio(frm.use_permissions).value == '1';
		frm.items.disabled = !use;
		frm.grant_create.disabled = !use;
		frm.grant_read.disabled = !use;
		frm.grant_use.disabled = !use;
		frm.grant_write.disabled = !use;
		frm.grant_delete.disabled = !use;
		frm.grant_set_owner.disabled = !use;
		frm.grant_set_permission.disabled = !use;
		frm.deny_create.disabled = !use;
		frm.deny_read.disabled = !use;
		frm.deny_use.disabled = !use;
		frm.deny_write.disabled = !use;
		frm.deny_delete.disabled = !use;
		frm.deny_set_owner.disabled = !use;
		frm.deny_set_permission.disabled = !use;
	}

	plugins.initPermissionLetters = function()
	{
		var frm = document.forms['plugin'];
		for (var i = 0; i < frm.itemTypes.length; i++)
		{
			plugins.updatePermissionLetters(frm.itemTypes[i]);
		}
		//usePermissionsOnClick();
	}
	
	// Show current permissions for the selected item
	plugins.itemTypesOnClick = function()
	{
		var frm = document.forms['plugin'];
		var itemType = frm.itemTypes.value;
		if (itemType)
		{
			var granted = parseInt(frm[itemType+'_granted'].value);
			var denied = parseInt(frm[itemType+'_denied'].value);
			var defined = parseInt(frm[itemType+'_defined'].value);
			
			plugins.updatePermissionCheckBoxes('create', Permission.CREATE, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('read', Permission.READ, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('use', Permission.USE, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('write', Permission.WRITE, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('delete', Permission.DELETE, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('set_owner', Permission.SET_OWNER, granted, denied, defined);
			plugins.updatePermissionCheckBoxes('set_permission', Permission.SET_PERMISSION, granted, denied, defined);
		}
	}

	/*
		Check/uncheck and enable/disable checkboxes depending on given permissions
	*/
	plugins.updatePermissionCheckBoxes = function(nameSuffix, permission, granted, denied, defined)
	{
		var frm = document.forms['plugin'];
		var isGranted = (permission & granted) > 0;
		var isDenied = (permission & denied) > 0;
		var isDefined = (permission & defined) > 0;
		
		frm['grant_'+nameSuffix].disabled = !isDefined;
		frm['deny_'+nameSuffix].disabled = !isDefined || isGranted;
		frm['grant_'+nameSuffix].checked = isGranted & isDefined;
		frm['deny_'+nameSuffix].checked = isDenied & isDefined;
	}

	
	/*
		Update the permission code string for the given list entry.
	*/
	plugins.updatePermissionLetters = function(listOption)
	{
		var frm = document.forms['plugin'];
		listOption = Doc.element(listOption);
		var itemType = listOption.value;
		if (itemType)
		{
			var granted = parseInt(frm[itemType+'_granted'].value);
			var denied = parseInt(frm[itemType+'_denied'].value);
			var defined = parseInt(frm[itemType+'_defined'].value);
			
			var s = '';
			s += plugins.getPermissionLetter('C', Permission.CREATE, granted, denied, defined);
			s += plugins.getPermissionLetter('R', Permission.READ, granted, denied, defined);
			s += plugins.getPermissionLetter('U', Permission.USE, granted, denied, defined);
			s += plugins.getPermissionLetter('W', Permission.WRITE, granted, denied, defined);
			s += plugins.getPermissionLetter('D', Permission.DELETE, granted, denied, defined);
			s += plugins.getPermissionLetter('O', Permission.SET_OWNER, granted, denied, defined);
			s += plugins.getPermissionLetter('P', Permission.SET_PERMISSION, granted, denied, defined);
			
			var text = listOption.text.replace(/\[.*\]/, '['+s+']');
			listOption.text = text; 
		}
	}
	
	plugins.getPermissionLetter = function(permissionLetter, permission, granted, denied, defined)
	{
		var letter = '';
		if ((defined & permission) > 0)
		{
			if ((granted & permission) > 0)
			{
				letter = permissionLetter.toUpperCase();
			}
			else if ((denied & permission) > 0)
			{
				letter = '-';
			}
			else
			{
				letter = permissionLetter.toLowerCase();
			}
		}
		return letter;
	}

	// When a user clicks on one of the permission checkboxes
	plugins.permissionOnClick = function(event)
	{
		var checkbox = event.currentTarget;
		
		var frm = document.forms['plugin'];
		// Make sure implied permissions are consistent
		plugins.ensureImpliedPermissions(checkbox.checked);
		// Calculate granted and denied permission values
		var granted = plugins.getCheckedPermissions('grant_');
		var denied = plugins.getCheckedPermissions('deny_');
		// Apply to the selected items and update their text
		for (var i = 0; i < frm.itemTypes.length; i++)
		{
			var item = frm.itemTypes[i];
			var itemType = item.value;
			if (item.selected && itemType)
			{
				frm[itemType+'_granted'].value = granted;
				frm[itemType+'_denied'].value = denied;
				plugins.updatePermissionLetters(item);
			}
		}
	}

	/*
		Get the permission for the checked checkboxes
		prefix should be "grant_" or "deny_"
	*/
	plugins.getCheckedPermissions = function(prefix)
	{
		var permissionCode = 0;
		var frm = document.forms['plugin'];
		permissionCode += frm[prefix+'create'].checked ? Permission.CREATE : 0;
		permissionCode += frm[prefix+'read'].checked ? Permission.READ : 0;
		permissionCode += frm[prefix+'use'].checked ? Permission.USE : 0;
		permissionCode += frm[prefix+'write'].checked ? Permission.WRITE : 0;
		permissionCode += frm[prefix+'delete'].checked ? Permission.DELETE : 0;
		permissionCode += frm[prefix+'set_owner'].checked ? Permission.SET_OWNER : 0;
		permissionCode += frm[prefix+'set_permission'].checked ? Permission.SET_PERMISSION : 0;
		return permissionCode;
	}

	
	/*
		Makes sure all checkboxes are in a consistent state with each other.
	*/
	plugins.ensureImpliedPermissions = function(checked)
	{
		var frm = document.forms['plugin'];
		
		// Grant permissions
		if (checked)
		{
			frm['grant_write'].checked = (frm['grant_write'].checked || frm['grant_delete'].checked || frm['grant_set_owner'].checked || frm['grant_set_permission'].checked);
			frm['grant_use'].checked = (frm['grant_use'].checked || frm['grant_write'].checked);
			frm['grant_read'].checked = (frm['grant_read'].checked || frm['grant_use'].checked);
		}
		else
		{
			frm['grant_use'].checked = (frm['grant_use'].checked && frm['grant_read'].checked);
			frm['grant_write'].checked = (frm['grant_write'].checked && frm['grant_use'].checked);
			frm['grant_delete'].checked = (frm['grant_delete'].checked && frm['grant_write'].checked);
			frm['grant_set_owner'].checked = (frm['grant_set_owner'].checked && frm['grant_write'].checked);
			frm['grant_set_permission'].checked = (frm['grant_set_permission'].checked && frm['grant_write'].checked);
		}
		
		// Deny permissions
		frm['deny_create'].disabled = frm['grant_create'].disabled || frm['grant_create'].checked;
		frm['deny_read'].disabled =frm['grant_read'].disabled || frm['grant_read'].checked;
		frm['deny_use'].disabled = frm['grant_use'].disabled || frm['grant_use'].checked;
		frm['deny_write'].disabled = frm['grant_write'].disabled || frm['grant_write'].checked;
		frm['deny_delete'].disabled = frm['grant_delete'].disabled || frm['grant_delete'].checked;
		frm['deny_set_owner'].disabled = frm['grant_set_owner'].disabled || frm['grant_set_owner'].checked;
		frm['deny_set_permission'].disabled = frm['grant_set_permission'].disabled || frm['grant_set_permission'].checked;
		if (checked)
		{
			if (!frm['deny_use'].disabled) frm['deny_use'].checked = frm['deny_use'].checked || frm['deny_read'].checked;
			if (!frm['deny_write'].disabled) frm['deny_write'].checked = frm['deny_write'].checked || frm['deny_use'].checked;
			if (!frm['deny_delete'].disabled) frm['deny_delete'].checked = frm['deny_delete'].checked || frm['deny_write'].checked;
			if (!frm['deny_set_owner'].disabled) frm['deny_set_owner'].checked = frm['deny_set_owner'].checked || frm['deny_write'].checked;
			if (!frm['deny_set_permission'].disabled) frm['deny_set_permission'].checked = frm['deny_set_permission'].checked || frm['deny_write'].checked;
		}
		else
		{
			if (!frm['deny_write'].disabled) frm['deny_write'].checked = frm['deny_write'].checked && !((!frm['deny_delete'].checked && !frm['deny_delete'].disabled) || (!frm['deny_set_owner'].checked && !frm['deny_set_owner'].disabled) || (!frm['deny_set_permission'].checked && !frm['deny_set_permission'].disabled));
			if (!frm['deny_use'].disabled) frm['deny_use'].checked = frm['deny_use'].checked && frm['deny_write'].checked;
			if (!frm['deny_read'].disabled) frm['deny_read'].checked = frm['deny_read'].checked && frm['deny_use'].checked;
		}
	}
	
	plugins.setRequestedPermissions = function()
	{
		var frm = document.forms['plugin'];
		// Reset all existing permissions
		for (var i = 0; i < frm.itemTypes.length; i++)
		{
			var itemType = frm.itemTypes[i].value;
			if (itemType)
			{
				frm[itemType+'_granted'].value = 0;
				frm[itemType+'_denied'].value = 255;
			}
		}
		
		// Set requested permissions
		var requested = Data.json('page-data', 'requested-permissions');
		for (var i = 0; i < requested.length; i++)
		{
			var rp = requested[i];
			var itemType = rp.itemType;
			frm[itemType+'_granted'].value = rp.granted;
			frm[itemType+'_denied'].value = rp.denied;
		}

		// Update the list
		plugins.initPermissionLetters();
	}
	
	return plugins;
}();

Doc.addElementInitializer(Plugins.initElements);
Doc.onLoad(Plugins.initPage);
