/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var BioWells = function()
{
	var biowells = {};
	
	/**
		Initialize the page.
	*/
	biowells.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			Buttons.addClickHandler('btnSave', biowells.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Biomaterial type
			Events.addEventHandler('biomaterial_type', 'change', biowells.bioMaterialTypeOnChange);
			
			// Biomaterial
			Buttons.addClickHandler('biomaterial_id.select', biowells.selectBioMaterial);
			Events.addEventHandler('biomaterial_id', 'base-selected', biowells.setBioMaterialCallback);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'BIOWELL'};
			var tableAttributes = {'table-id': 'biowells'};
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
			
			TabControl.setSwitchFunction('main', biowells.switchTab);
		}
	}
	
	biowells.switchTab = function(event)
	{
		var target = event.target;
		var tabId = target.id.substring(5);

		var frm = document.forms['biowells'];
		var bioPlateId = frm.bioplate_id.value;
		
		var url;
		if (tabId == 'events')
		{
			url = '../events/index.jsp?ID='+App.getSessionId();
			url += '&bioplate_id='+bioPlateId;
		}
		else
		{
			url = '../index.jsp?ID='+App.getSessionId();
			url += '&cmd=ViewItem&item_id='+bioPlateId;
			url += '&tab='+tabId;
		}
		location.replace(url);
	}
	
	biowells.validateBioWell = function()
	{
		return true;
	}
	
	biowells.save = function()
	{
		var frm = document.forms['biowell'];
		if (biowells.validateBioWell())
		{
			frm.biomaterial_type.disabled = false;
			frm.submit();
		}
	}
	
	biowells.bioMaterialTypeOnChange = function()
	{
		var frm = document.forms['biowell'];
		var list = frm.biomaterial_id;
		list.selectedIndex = 0;
		list.remove(1);
	}

	biowells.selectBioMaterial = function()
	{
		var frm = document.forms['biowell'];		
		var type = frm.biomaterial_type.value;
		if (!type)
		{
			Forms.showNotification(frm.biomaterial_type, 'You must first select what type of biomaterial');
			return;
		}			
		var url = '';
		if (frm.biomaterial_id.length > 1) 
		{
			var id = Math.abs(parseInt(frm.biomaterial_id[1].value));				
			url += '&item_id='+id;
		}
		url += '&resetTemporary=1&tmpfilter:STRING:bioWell='+escape('=');
		var bioMaterialSubtype = Data.get('biomaterial_id', 'biomaterial-subtype');
		if (bioMaterialSubtype)
		{
			url += '&tmpfilter:INT:itemSubtype='+bioMaterialSubtype;
		}
		Dialogs.selectItem(type, 'biomaterial_id', 0, url);
	}
	
	biowells.setBioMaterialCallback = function(event)
	{
		var frm = document.forms['biowell'];
		var list = frm.biomaterial_id;
		if (list.length < 2 || list[1].value == '0')
		{
			Forms.addListOption(list, 1, new Option());
		}
		list[1].value = event.detail.id;
		list[1].text = event.detail.name;
		list.selectedIndex = 1;
	}

	
	return biowells;
}();

Doc.onLoad(BioWells.initPage);
