<%-- $Id: place_on_plate.jsp 6611 2014-11-20 13:53:55Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.BioPlateEvent"
	import="net.sf.basedb.core.BioPlateEventType"
	import="net.sf.basedb.core.MeasuredBioMaterial"
	import="net.sf.basedb.core.Hardware"
	import="net.sf.basedb.core.Protocol"
	import="net.sf.basedb.core.ItemSubtype"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.SystemItems"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.ItemQuery"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.query.Restrictions"
	import="net.sf.basedb.core.query.Hql"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.Formatter"
	import="net.sf.basedb.clients.web.formatter.FormatterFactory"
	import="net.sf.basedb.clients.web.formatter.FormatterSettings"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.util.HTML"
	import="java.util.List"
	import="java.util.Date"
	import="java.util.Collections"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%@ taglib prefix="tbl" uri="/WEB-INF/table.tld" %>
<%
final Item itemType = Item.valueOf(request.getParameter("itemType"));
final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final ItemContext cc = sc.getCurrentContext(Item.BIOPLATEEVENT, BioPlateEventType.PLACE_ON_PLATE);
final String ID = sc.getId();
final float scale = Base.getScale(sc);
final DbControl dc = sc.newDbControl();
try
{
	// Load recently used items
	List<Hardware> recentHardware = (List<Hardware>)cc.getRecent(dc, Item.HARDWARE);
	List<Protocol> recentProtocols = (List<Protocol>)cc.getRecent(dc, Item.PROTOCOL);
	
	ItemQuery<MeasuredBioMaterial> query = (ItemQuery<MeasuredBioMaterial>)cc.getQuery();
	List<MeasuredBioMaterial> bioMaterial = Collections.emptyList();
	ItemSubtype commonSubtype = null;
	if (query != null)
	{
		query.restrict(Restrictions.eq(Hql.property("bioWell"), null));
		query.setMaxResults(-1);
		query.setFirstResult(0);
		bioMaterial = query.list(dc);
		cc.setQuery(null);
	}
	
	final Formatter<Date> dateFormatter = FormatterFactory.getDateFormatter(sc);
	final String dateFormat = FormatterSettings.getDateFormat(sc);
	final String jsDateFormat = HTML.javaScriptEncode(dateFormat);
	final String htmlDateFormat = HTML.encodeTags(dateFormat);
	final boolean autoSelectUnmappedItem = Values.getBoolean(sc.getUserClientSetting("place-on-plate.auto-select-unmapped-item"), true);
	
	// Find out if all selected biomaterial has a common subtype
	boolean first = true;
	for (MeasuredBioMaterial bm : bioMaterial)
	{
		if (first)
		{
			commonSubtype = bm.getItemSubtype();
			first = false;
		}
		else if (commonSubtype != null)
		{
			if (!commonSubtype.equals(bm.getItemSubtype()))
			{
				commonSubtype = null;
				break;
			}
		}
	}

	ItemSubtype protocolType = null;
	ItemSubtype hardwareType = null;
	if (commonSubtype != null)
	{
		protocolType = commonSubtype.getRelatedSubtype(Item.PROTOCOL);
		hardwareType = commonSubtype.getRelatedSubtype(Item.HARDWARE);
	}
	if (protocolType == null)
	{
		String defaultId = Protocol.getDefaultSystemId(itemType);
		if (defaultId != null) protocolType = ItemSubtype.getById(dc, SystemItems.getId(defaultId));
	}
	if (hardwareType == null)
	{
		String defaultId = Hardware.getDefaultSystemId(itemType);
		if (defaultId != null) hardwareType = ItemSubtype.getById(dc, SystemItems.getId(defaultId));
	}
	%>
	<base:page type="popup" title="Place biomaterial on plate">
	<base:head scripts="plate.js,js-draw.js,~wizards.js,~place_on_plate.js" styles="plate.css,toolbar.css" />
	<base:body>
		<div id="canvas" style="position: absolute; z-index: 99;"></div>
		<h1>Place biomaterial on plate <base:help helpid="bioplateevent.place-on-plate" /></h1>
		
		<div id="page-data" class="datacontainer"
			data-protocol-type="<%=protocolType != null ? protocolType.getId() : 0 %>"
			data-hardware-type="<%=hardwareType != null ? hardwareType.getId() : 0 %>"
			data-biomaterial-type="<%=itemType.getValue()%>"
			data-common-subtype="<%=commonSubtype != null ? commonSubtype.getId() : 0 %>"
		></div>
		
		<form name="main" action="index.jsp" method="post">
			<input type="hidden" name="ID" value="<%=ID%>">
			<input type="hidden" name="cmd" value="PlaceOnPlate">
			<input type="hidden" name="itemType" value="<%=itemType.name()%>">
			<input type="hidden" name="plate_id" value="">
			<input type="hidden" name="rows" value="">
			<input type="hidden" name="columns" value="">
		
		<div class="content">
		<%
		if (bioMaterial.size() == 0)
		{
			%>
			<div class="absolutefull bg-filled-50 bottomborder">
			<div class="messagecontainer error">
			No biomaterial selected. Please close this window and select one or more items that
			are not already placed on a plate.
			</div>
			</div>
			<%
		}
		else
		{
			%>
			<div class="absolutefull" style="height: 10em;">
				<div class="absolutefull" style="width: 50%; bottom: 2em;">
					<table class="fullform input100 smaller">
					<tr>
						<th>Event name</th>
						<td><input class="text required" type="text" name="name" 
							value="Place biomaterial on plate" 
							maxlength="<%=BioPlateEvent.MAX_NAME_LENGTH%>"></td>
					</tr>
					<tr>
						<th>Event date</th>
						<td>
							<input class="text" type="text" name="event_date" id="event_date" style="width: 15em;"
								value="<%=HTML.encodeTags(dateFormatter.format(new Date()))%>" 
								maxlength="20" title="Enter date in format: <%=htmlDateFormat%>">
							<base:calendar textarea="event_date" title="Event date" />
						</td>
					</tr>
					<tr>
						<th>Protocol</th>
						<td>
							<base:select 
								id="protocol_id"
								clazz="selectionlist"
								required="false"
								current="<%=null%>"
								recent="<%=recentProtocols%>"
							/>
						</td>
					</tr>
					<tr>
						<th>Hardware</th>
						<td>
							<base:select 
								id="hardware_id"
								clazz="selectionlist"
								required="false"
								current="<%=null%>"
								recent="<%=recentHardware%>"
							/>
						</td>
					</tr>
					</table>
				</div>
				<div class="absolutefull" style="left: 50%; width: 50%;">
					<table style="width: 100%;">
					<tr>
						<td><b>Description</b><br>
							<textarea class="text" rows="4" name="description" id="description" style="width: calc(100% - 10px);"
							></textarea></td>
						<td style="width: 20px;">
							<base:zoom textarea="description" title="Description" />
						</td>
					</tr>
					</table>
				</div>
				<div class="absolutefull" style="top: auto; height: 2em;">
					<tbl:toolbar id="toolbar.mappings" subclass="topborder bottomborder bg-filled-50" style="height: 100%;">
						<tbl:button 
							id="btnSelectPlate"
							title="Select plate&hellip;" 
							image="place_on_plate.png" 
							tooltip="Select the destination plate"
						/>
						<tbl:button 
							id="btnClearMapping"
							title="Clear" 
							image="cancel.png" 
							tooltip="Clear all mapped wells"
						/>
						<tbl:button 
							id="btnPlaceByRow"
							title="Place by row" 
							image="place_by_row.png" 
							tooltip="Place remaining items; start with rows"
						/>
						<tbl:button 
							id="btnPlaceByColumn"
							title="Place by column" 
							image="place_by_column.png" 
							tooltip="Place remaining items; start with columns"
						/>
					</tbl:toolbar>
				</div>
			</div>
			
			<div class="absolutefull bottomborder" style="top: 10em;">
				<div class="absolutefull rightborder" style="width: 280px;">
					<div class="absolutefull bg-filled-100" style="height: 2em;">
						<div class="padded">
						<b>Items to place</b> <%=commonSubtype != null ? "<span class=\"itemsubtype\">[" + HTML.encodeTags(commonSubtype.getName()) + "]</span>" : "" %>
						</div>
					</div>

					<div id="itemlist" class="absolutefull biomateriallist topborder" 
						style="top: 2em; bottom: 2em;">
							<table style="width: 100%; height: 100%; border-collapse: separate;">
							<%
							for (MeasuredBioMaterial bm : bioMaterial)
							{
								String itemId = Integer.toString(bm.getId());
								String fullLabel = bm.getName();
								String label = Values.trimString(fullLabel, 30);
								%>
								<tr id="itemlist.<%=itemId%>" class="item" 
									data-item-id="<%=itemId %>"
									data-name="<%=HTML.encodeTags(bm.getName()) %>"
									style="height: 1.5em;" title="<%=fullLabel == label ? "" : HTML.encodeTags(fullLabel)%>"
									>
									<td class="info bg-filled-100" id="itemlist.<%=itemId%>.info"></td>
									<td class="label interactable" id="itemlist.<%=itemId%>.label"><%=HTML.encodeTags(label)%></td>
								</tr>
								<%
							}
							%>
							<tr style="height: auto;" class="item">
								<td class="info"></td>
								<td></td>
							</tr>
							</table>
					</div>
					<div class="absolutefull bg-filled-100 topborder" style="top: auto; height: 2em;">
							<input type="checkbox" name="autoSelectUnmappedItem" id="autoSelectUnmappedItem"
								<%=autoSelectUnmappedItem ? "checked" : ""%> value="1"><label for="autoSelectUnmappedItem">Auto-select next unmapped item</label>
					</div>
				</div>
				<div class="absolutefull" style="left: 280px;">
					<div class="absolutefull bg-filled-100" style="height: 2em;">
						<div class="padded">
						<b>Destination plate:</b> <span id="plate.name"><i>no plate selected</i></span>
						</div>
					</div>
					<div class="absolutefull topborder" style="top: 2em;">
						<table style="margin: auto; height: 100%;"><tr><td>
						<div id="plate"></div>
						</td></tr></table>
					</div>
				</div>
			</div>
			<%
		}
		%>
	</div>
	</form>
		
	<base:buttongroup subclass="dialogbuttons">
		<base:button id="btnSave" disabled="<%=bioMaterial.size() == 0 %>" title="Save" />
		<base:button id="close" title="Cancel" />
	</base:buttongroup>
		
	</base:body>
	</base:page>
	<%
}
finally
{
	if (dc != null) dc.close();
}
%>