/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var ArraySlides = function()
{
	var arraySlides = {};
	
	/**
		Initialize the page.
	*/
	arraySlides.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', arraySlides.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabActivateListener('settings.annotations', Annotations.autoLoadEditFrame);
			TabControl.addTabActivateListener('settings.inheritedAnnotations', arraySlides.loadInheritedAnnotationsFrame);
			TabControl.addTabValidator('settings.info', arraySlides.validateArraySlide);
			
			// Array batch
			Buttons.addClickHandler('arraybatch_id.select', arraySlides.selectArrayBatch);
			Events.addEventHandler('arraybatch_id', 'base-selected', arraySlides.setArrayBatchCallback);
			
			// Batch order
			Events.addEventHandler('batch_index', 'keypress', Events.integerOnly);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'ARRAYSLIDE', 'item-id': itemId};
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			TabControl.addTabActivateListener('main.overview', Overview.loadOnce);
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'ARRAYSLIDE'};
			var tableAttributes = {'table-id': 'slides'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnNewItemsWizard', arraySlides.newSlidesWizard, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
		
	arraySlides.newSlidesWizard = function()
	{
		var url = 'index.jsp?ID='+App.getSessionId();
		url += '&cmd=NewItems';
		Dialogs.openPopup(url, 'NewArraySlides', 750, 500);
	}
	
	arraySlides.validateArraySlide = function()
	{
		var frm = document.forms['slide'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.arraybatch_id && !frm.arraybatch_id.value)
		{
			Forms.showNotification('arraybatch_id.select', 'You must select an array batch');
			return false;
		}
		return true;
	}

	arraySlides.save = function()
	{
		var frm = document.forms['slide'];
		if (TabControl.validateActiveTab('settings'))
		{
			Annotations.saveModifiedAnnotationsToForm(frm);
			Annotations.saveInheritedAnnotationsToForm(frm);
			frm.submit();
		}
	}
	
	arraySlides.loadInheritedAnnotationsFrame = function()
	{
		Annotations.autoLoadInheritFrame(arraySlides.getParents());
	}
	
	arraySlides.getParents = function()
	{
		var frm = document.forms['slide'];
		var parents = new Array();
		if (frm.arraybatch_id)
		{
			var batchId = Math.abs(parseInt(frm.arraybatch_id[frm.arraybatch_id.selectedIndex].value));
			if (batchId > 0) parents[parents.length] = 'ARRAYBATCH:'+batchId;
		}
		return parents;
	}

	arraySlides.selectArrayBatch = function()
	{
		var frm = document.forms['slide'];
		var url = '';
		if (frm.arraybatch_id.length > 0) 
		{
			url += '&item_id='+frm.arraybatch_id[0].value;
		}
		Dialogs.selectItem('ARRAYBATCH', 'arraybatch_id', 0, url);
	}
	
	arraySlides.setArrayBatchCallback = function(event)
	{
		var frm = document.forms['slide'];
		var list = frm.arraybatch_id;
		if (list.length < 1 || list[0].value == '0')
		{
			Forms.addListOption(list, 0, new Option());
		}
		list[0].value = event.detail.id;
		list[0].text = event.detail.name;
		list.selectedIndex = 0;
	}

	
	return arraySlides;
}();

Doc.onLoad(ArraySlides.initPage);
