/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var ReporterLists = function()
{
	var reporterlists = {};
	var selectedReporters = null;
	
	/**
		Initialize the page.
	*/
	reporterlists.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', reporterlists.save);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', reporterlists.validateReporterList);
			
			// Min count
			Events.addEventHandler('minCount', 'keypress', Events.integerOnly);
			Events.addEventHandler('minCount', 'focus', reporterlists.minCountOnFocus);
			
			var addReportersForm = Data.get('page-data', 'add-reporters');
			if (addReportersForm)
			{
				var frm = document.forms['reporterList'];
				selectedReporters = window.opener.Table.getSelected(addReportersForm);
				var lblSelected = Doc.element('lblSelected');
				if (selectedReporters.length == 0)
				{
					frm.which[0].disabled = true;
					lblSelected.disabled = true;
					Doc.addClass(lblSelected, 'disabled');
				}
				else
				{
					frm.which[0].checked = true;
				}
			}
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'REPORTERLIST', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			
			Buttons.addClickHandler('btnMergeUnion', reporterlists.merge, attributes);
			Buttons.addClickHandler('btnMergeIntersection', reporterlists.merge, attributes);
			Buttons.addClickHandler('btnMergeComplement', reporterlists.merge, attributes);
			
			TabControl.addTabActivateListener('main.reporters', reporterlists.viewReporters);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'REPORTERLIST'};
			var tableAttributes = {'table-id': 'reporterLists'};
			Buttons.addClickHandler('btnNewItem', reporterlists.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);

			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}
	
	reporterlists.newItem = function()
	{
		var controller = Items.getController('REPORTERLIST');
		Table.submitToPopup('reporterLists', 'NewItem', controller.width, controller.height);
	}

	reporterlists.merge = function(event)
	{
		var url = 'merge.jsp?ID='+App.getSessionId();
		url += '&item_id='+Data.get(event.currentTarget, 'item-id');
		url += '&mergeType='+Data.get(event.currentTarget, 'merge-type');
		Dialogs.openPopup(url, 'MergeReporterLists', 600, 400);
	}

	reporterlists.viewReporters = function()
	{
		var url = 'reporters/index.jsp?ID='+App.getSessionId();
		url += '&cmd=List&reporterlist_id='+Data.get('page-data', 'item-id');
		location.replace(url);
	}
	
	
	reporterlists.validateReporterList = function()
	{
		var frm = document.forms['reporterList'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (frm.minCount)
		{
			var checked = Forms.getCheckedRadio(frm.create);
			if (checked.value == 'count')
			{
				if (!Numbers.isInteger(frm.minCount.value))
				{
					Forms.showNotification(frm.minCount, 'You must specify the number of reporter lists');
					return false;
				}
				else
				{
					var minCount = parseInt(frm.minCount.value);
					var numLists = Data.int('page-data', 'num-selected-lists');
					if (minCount < 1 || minCount > numLists)
					{
						Forms.showNotification(frm.minCount, 'You must specify a value between 1 and '+ numLists);
						return false;
					}
				}
			}
		}
		return true;
	}
	
	reporterlists.save = function()
	{
		var frm = document.forms['reporterList'];
		if (TabControl.validateActiveTab('settings'))
		{
			if (frm.which) Forms.addHidden(frm, 'items', selectedReporters.join(','));
			frm.submit();
		}
	}

	reporterlists.minCountOnFocus = function()
	{
		var frm = document.forms['reporterList'];
		Forms.checkRadio(frm.create, 'count');
	}
	
	return reporterlists;
}();

Doc.onLoad(ReporterLists.initPage);

